Function SelectedSize()
	Select SelectedGadgetItem(cbDim)
		Case 0 Return 33;
		Case 1 Return 65;
		Case 2 Return 129;
		Case 3 Return 257;
		Case 4 Return 513;
	EndSelect
	
	Return 127;
EndFunction

Function Generate()
	Local h:THeightField;
	h = THeightField.Create(SelectedSize());
	h.DiamondSquareFractal(Float(GadgetText(tfBlend)), Float(GadgetText(tfFeature)));
	Image = LoadImage(h.ToPixmap(), 0);
EndFunction

Type THeightField
	Field size
	Field height:Short[,]

	Function Create:THeightField(size)
		Local h:THeightfield = New THeightField;
		Local height:Short[size, size];
		map = New Byte[size,size];
		map_sizex = size - 1;
		map_sizey = size - 1;
		map_name = "Generated Map";
		ClearList(map_entity);
		h.height = height;
		h.size = size;
		Return h;
	EndFunction
	
	Method Copy:THeightField()
		Local h:THeightfield = New THeightField;
		Local height:Short[size, size];
		Local x, y;
		h.height = height;
		h.size = size
		MemCopy(h.height, height, size * size * 2);
		Return h;
	EndMethod
	
	Method Blend(h:THeightfield, b#=0.5)
		Local x, y;
		For x = 0 To size - 1
			For y=0 To size - 1
				height[x, y] = height[x, y] * b + h.height[x, y] * (1.0 - b);
			Next
		Next
	EndMethod
	
	Method Noise()
		Local x, y;
		For x = 0 To size - 1
			For y = 0 To size - 1
				height[x, y] = Rand(65536);
			Next
		Next
	EndMethod
	
	Method Flatten(h#=0)
		Local x, y;
		For x = 0 To size - 1
			For y = 0 To size - 1
				height[x, y] = h * 65536;
			Next
		Next
	EndMethod
	
	Method DiamondSquareFractal(blend#=0.5, featuresize#=64)
		Local x, y, res;
		height[0,0] = Rand(0, 65536);
		height[size - 1,0] = Rand(0, 65536);
		height[0,size - 1] = Rand(0, 65536);
		height[size- 1,size - 1] = Rand(0, 65536);
		res = size;
		Local iteration, passes = 1, gridsize = size;
		Repeat
			iteration :+ 1;
			
			' Diamond
			For x = 0 To passes - 1
				For y = 0 To passes - 1
					DiamondFractal(x * (gridsize - 1), y * (gridsize - 1), res, blend, featuresize);
				Next
			Next
			
			' Square
			For x=0 To passes-1
				For y=0 To passes-1
					SquareFractal(x * (gridsize - 1) + (res - 1) / 2, y * (gridsize - 1), res, blend, featuresize)
					SquareFractal(x * (gridsize - 1), y * (gridsize - 1) + (res - 1) / 2, res, blend, featuresize)
					SquareFractal(x * (gridsize - 1) + (res - 1), y * (gridsize - 1) + (res - 1) / 2, res, blend, featuresize)
					SquareFractal(x * (gridsize - 1) + (res - 1) / 2, y * (gridsize - 1) + (res - 1), res, blend, featuresize)
				Next
			Next
			
			res = (res + 1) / 2;
			passes :* 2;
			If res = 2 Then Exit;
			gridsize = (gridsize + 1) / 2;
		Forever
	EndMethod

	Method DiamondFractal(x0, y0, d, blend#, featuresize#)
		Local hd, x1, y1;
		x1 = x0 + d - 1;
		y1 = y0 + d - 1;
		hd = (d + 1) / 2;
		Local i, avg#;
		Local distblend# = Min(Float(d) / featuresize, 1.0);
		blend :* distblend;
		
		height[x0 + hd - 1,y0 + hd - 1] = (height[x0, y0] + height[x0, y1] + height[x1, y0] + height[x1, y1]) / 4;
	EndMethod
	
	Method SquareFractal(x0, y0, d, blend#, featuresize#)
		Local hd, x1, y1;
		x1 = x0 + d - 1;
		y1 = y0 + d - 1;
		hd = (d - 1) / 2;
		Local i = 0;
		Local avg# = 0;
		
		If x0 - hd => 0 Then
			i :+ height[x0 - hd, y0];
			avg :+ 1;
		EndIf

		If x0 + hd <= size - 1 Then
			i :+ height[x0 + hd, y0];
			avg :+ 1;
		EndIf

		If y0 - hd => 0 Then
			i :+ height[x0, y0 - hd];
			avg :+ 1;
		EndIf

		If y0 + hd <= size - 1 Then
			i :+ height[x0, y0 + hd];
			avg :+ 1;
		EndIf
		
		Local distblend# = Min(Float(d) / featuresize, 1.0);
		blend :* distblend;
		height[x0, y0] = i / avg * (1.0 - blend) + Rand(65535) * blend;
	EndMethod
	
	Method Multiply(m#)
		Local x, y;
		For x = 0 To size - 1
			For y=0 To size - 1
				height[x, y] :* m;
			Next
		Next
	EndMethod
	
	Method ToPixmap:TPixmap()
		Local ent:TEntity;
		Local x, y, h, r, g, b, z;
		Local p:TPixmap = CreatePixmap(size, size, PF_RGB888)
		For x = 0 To size - 1
			For y = 0 To size - 1
				h = height[x, y] / 65536.0 * 255.0;
				If h < 64 Then
					r = 0;
					g = 0;
					b = 64;
					
					' Water
					map[x,y] = 34;
				ElseIf h < 128 Then
					r = 0;
					g = 0;
					b = h;
					
					' Water
					map[x,y] = 34;
				ElseIf h < 150 Then
					If Rand(0,100) < 2 Then
						r = 0;
						g = 30;
						b = 0;
						
						' Beach Tree
						map[x,y] = 6;
						ent = TEntity.Create("", 29, x, y, "");
						ent.ints[0] = 0;
						ent.ints[1] = 255;
						ent.ints[2] = 255;
						ent.ints[3] = 255;
						ent.ints[4] = Rand(0,359);
						ent.strs[0] = "1";
						ent.strs[1] = "1.0";
					Else
						z = ((Float(h - 128) / 32) * 255);
						r = 176 - z;
						g = h;
						b = 0;
						
						' Sand
						z = Rand(0,100);
						If z < 3 Then
							map[x,y] = 24;
						ElseIf z < 10
							map[x,y] = 8;
						ElseIf z < 20
							map[x,y] = 7;
						Else
							map[x,y] = 6;
						EndIf
					EndIf
				ElseIf h < 175 Then
					If Rand(0,100) < 5 Then
						r = 0;
						g = 25;
						b = 0;
						
						' Tree
						map[x,y] = 1;
						
						ent = TEntity.Create("", 29, x, y, "");
						ent.ints[0] = 1;
						ent.ints[1] = 255;
						ent.ints[2] = 255;
						ent.ints[3] = 255;
						ent.ints[4] = Rand(0,359);
						ent.strs[0] = "1";
						ent.strs[1] = "1.0";
					Else
						r = 0;
						g = h;
						b = 0;
						
						' Grass
						z = Rand(0,100);
						If z < 3 Then
							map[x,y] = 4;
						ElseIf z < 10
							map[x,y] = 3;
						ElseIf z < 20
							map[x,y] = 2;
						Else
							map[x,y] = 1;
						EndIf
					EndIf
				ElseIf h < 200 Then
					If Rand(0,100) < 3 Then
						r = 0;
						g = 100;
						b = 0;
						
						' High Land Tree
						map[x,y] = 1;
						ent = TEntity.Create("", 29, x, y, "");
						ent.ints[0] = 1;
						ent.ints[1] = 255;
						ent.ints[2] = 255;
						ent.ints[3] = 255;
						ent.ints[4] = Rand(0,359);
						ent.strs[0] = "1.5";
						ent.strs[1] = "1.0";
					Else
						z = (Float(h - 150) / 105) * 255;
						r = z;
						g = h;
						b = z;
						
						' High Land
						z = Rand(0,100);
						If z < 3 Then
							map[x,y] = 4;
						ElseIf z < 10
							map[x,y] = 3;
						ElseIf z < 20
							map[x,y] = 2;
						Else
							map[x,y] = 1;
						EndIf
					EndIf
				Else
					z = (Float(h - 150) / 105) * 255;
					r = z;
					g = h;
					b = z;
					
					' Mountain
					map[x,y] = 104;
				EndIf
				p.WritePixel(x, y, b + (g Shl 8) + (r Shl 16));
			Next
		Next
		Return p
	EndMethod
	
	Method FromPixmap(pixmap:TPixmap)
		Local x, y, px, py;
		For x = 0 To size - 1
			For y = 0 To size - 1
				px = Min(x, pixmap.width - 1)
				py = Min(y, pixmap.height - 1)			
				height[x, y] = ((pixmap.ReadPixel(px, py) & $00FF0000) Shr 16) / 255.0 * 65536;
			Next
		Next
	EndMethod
EndType