' Map Data
Global map_name$ = "";
Global map_sizex% = 0;
Global map_sizey% = 0;
Global map_code$ = "";

Global map:Byte[,];
Global map_data:TTile[,];
Global map_modifier:Byte[,];
Global map_entity:TList = CreateList();
Global map_blending = False;

' Author
Global map_authorid% = 9102;
Global map_authornick$ = "Map Generator";
Global map_check_ms% = 0;

' Tileset
Global map_tls_name$ = "cs2dnorm.bmp";
Global map_tls_cnt% = 0;
Global map_tls_mode:Int[];

' Background
Global map_back_scr_tile% = 0;
Global map_back$ = "";
Global map_back_scr_x% = 0;
Global map_back_scr_y% = 0;
Global map_back_r% = 0;
Global map_back_g% = 0;
Global map_back_b% = 0;

' Error
Global map_loaderror$ = "";
Global map_saveerror$ = "";

Function saveMap(path$)
	Local stream:TStream = WriteStream(path);
	
	map_saveerror = "";
	
	If Not stream Then
		map_saveerror = "Failed to save map!";
		Return False;
	EndIf
	
	' Header
	stream.WriteLine("Unreal Software's Counter-Strike 2D Map File (max)");
	
	' Data
	Local i%, x%, y%;
	Local bytes:Byte[10];
	Local ints:Int[10];
	Local lines:String[10];
	
	bytes[0] = map_back_scr_tile;
	bytes[1] = map_blending;
	ints[0] = MilliSecs();
	ints[1] = map_authorid + 51;
	lines[0] = map_authornick;
	
	For i = 1 To 10
		stream.WriteByte(bytes[i - 1]);
	Next
	For i = 1 To 10
		stream.WriteInt(ints[i - 1]);
	Next
	For i = 1 To 10
		stream.WriteLine(lines[i - 1]);
	Next
	
	' Map Data
	stream.WriteLine(""); ' map code
	stream.WriteLine(map_tls_name);
	stream.WriteByte(map_tls_cnt);
	stream.WriteInt(map_sizex);
	stream.WriteInt(map_sizey);
	stream.WriteLine(map_back);
	stream.WriteInt(map_back_scr_x);
	stream.WriteInt(map_back_scr_y);
	stream.WriteByte(map_back_r);
	stream.WriteByte(map_back_g);
	stream.WriteByte(map_back_b);
	stream.WriteLine("ed.erawtfoslaernu"); ' map code
	
	' Tile Modes
	For i = 0 To map_tls_cnt
		stream.WriteByte(map_tls_mode[i]);
	Next
	
	' Map Tiles
	For x = 0 To map_sizex
		For y = 0 To map_sizey
			stream.WriteByte(map[x, y]);
		Next
	Next
	
	' Map Blending
	If map_blending Then
		For x = 0 To map_sizex
			For y = 0 To map_sizey
				stream.WriteByte(map_modifier[x, y]);
				If map_modifier[x, y] & (64 + 128) > 0 Then
					If map_modifier[x, y] & (64 + 128) = 192 Then
						stream.WriteLine("");
					ElseIf map_modifier[x, y] & (64 + 128) = 64 Then
						stream.WriteByte(map_data[x, y].frame);
					Else
						stream.WriteByte(map_data[x, y].r);
						stream.WriteByte(map_data[x, y].g);
						stream.WriteByte(map_data[x, y].b);
						stream.WriteByte(map_data[x, y].overlay);
					EndIf
				EndIf
			Next
		Next
	EndIf
	
	' Entities
	Local ent_count = map_entity.Count();
	Local ent:TEntity;
	stream.WriteInt(ent_count);
	
	For ent = EachIn(map_entity)
		stream.WriteLine(ent.name);
		stream.WriteByte(ent.etype);
		stream.WriteInt(ent.x);
		stream.WriteInt(ent.y);
		stream.WriteLine(ent.trigger);
		
		For x = 0 To 9
			stream.WriteInt(ent.ints[x]);
			stream.WriteLine(ent.strs[x]);
		Next
	Next
	
	CloseStream(stream);
EndFunction

Function loadMap(path$, name$)
	Local stream:TStream = ReadStream(path + name + ".map");
	
	map_loaderror = "";
	
	If Not stream Then
		map_loaderror = "Map doesn't exist!";
		Return False;
	EndIf
	
	Local header$;
	header = stream.ReadLine();
	
	If header.Find("Unreal Software's Counter-Strike 2D Map File") = -1 Then
		map_loaderror = "Unkown map type!";
		CloseStream(stream);
		Return False;
	EndIf
	
	' Data
	Local i%, x%, y%;
	Local bytes:Byte[10];
	Local ints:Int[10];
	Local lines:String[10];
	
	For i = 1 To 10
		bytes[i - 1] = stream.ReadByte();
	Next
	For i = 1 To 10
		ints[i - 1] = stream.ReadInt();
	Next
	For i = 1 To 10
		lines[i - 1] = stream.ReadLine();
	Next
	
	map_back_scr_tile = bytes[0]; ' Get background scroll mode
	map_check_ms = ints[0]; ' Milliseconds since computer has been turned on
	map_authorid = ints[1] - 51; ' Get author usgn
	map_authornick = lines[0]; ' Get authour nick
	
	' Map Data
	map_code = stream.ReadLine();
	map_name = name;
	map_tls_name = stream.ReadLine();
	map_tls_cnt = stream.ReadByte();
	map_sizex = stream.ReadInt();
	map_sizey = stream.ReadInt();
	map_back = stream.ReadLine();
	map_back_scr_x = stream.ReadInt();
	map_back_scr_y = stream.ReadInt();
	map_back_r = stream.ReadByte();
	map_back_g = stream.ReadByte();
	map_back_b = stream.ReadByte();
	header = stream.ReadLine();
	
	If header <> "ed.erawtfoslaernu" Then
		map_loaderror = "Map is not valid!";
		CloseStream(stream);
		Return False;
	EndIf
	
	map_tls_mode = New Int[map_tls_cnt + 1];
	For i = 0 To map_tls_cnt
		map_tls_mode[i] = stream.ReadByte();
	Next
	
	' Map Tiles
	map = New Byte[map_sizex + 1, map_sizey + 1];
	For x = 0 To map_sizex
		For y = 0 To map_sizey
			map[x, y] = stream.ReadByte();
		Next
	Next
	
	' Map Blending
	map_blending = bytes[1];
	If bytes[1] Then
		map_modifier = New Byte[map_sizex + 1, map_sizey + 1];
		map_data = New TTile[map_sizex + 1, map_sizey + 1];
		For x = 0 To map_sizex
			For y = 0 To map_sizey
				map_modifier[x, y] = stream.ReadByte();
				If map_modifier[x, y] & (64 + 128) > 0 Then
					If map_modifier[x, y] & (64 + 128) = 192 Then
						stream.ReadLine();
					ElseIf map_modifier[x, y] & (64 + 128) = 64 Then
						map_data[x, y] = TTile.Create(stream.ReadByte());
					Else
						Local r, g, b, overlay;
						r = stream.ReadByte();
						g = stream.ReadByte();
						b = stream.ReadByte();
						overlay = stream.ReadByte();
						map_data[x, y] = TTile.Create(0, r, g, b, overlay);
					EndIf
				EndIf
			Next
		Next
	EndIf
	
	' Entities
	Local ent_count = stream.ReadInt();
	Local Ent:TEntity;
	map_entity.Clear();
	
	For i = 1 To ent_count
		If Not Eof(stream) Then
			Ent = TEntity.Create();
			Ent.name = stream.ReadLine();
			Ent.etype = stream.ReadByte();
			Ent.x = stream.ReadInt();
			Ent.y = stream.ReadInt();
			Ent.trigger = stream.ReadLine()
			For x = 0 To 9
				Ent.ints[x] = stream.ReadInt();
				Ent.strs[x] = stream.ReadLine();
			Next
		Else
			Exit;
		EndIf
	Next
	
	CloseStream(stream);
	
	Return True;
EndFunction

Type TEntity
	Field name$
	Field etype:Byte
	Field x%, y%
	Field trigger$
	Field ints:Int[10]
	Field strs:String[10]
	
	Function Create:TEntity(name$ = "", etype = 0, x = 0, y = 0, trigger$ = "")
		Local Ent:TEntity = New TEntity;
		Ent.name = name;
		Ent.etype = etype;
		Ent.x = x;
		Ent.y = y;
		Ent.trigger = trigger;
		map_entity.AddLast(Ent);
		Return Ent;
	EndFunction
EndType

Type TTile
	Field frame:Byte
	Field r:Byte
	Field g:Byte
	Field b:Byte
	Field overlay:Byte
	Function Create:TTile(frame = 0, r = 0, g = 0, b = 0, overlay = 0)
		Local T:TTile = New TTile;
		T.frame = frame;
		T.r = r;
		T.g = g;
		T.b = b;
		T.overlay = overlay;
		Return T;
	EndFunction
	
	Method Copy:TTile()
		Local T:TTile = New TTile;
		T.frame = frame;
		T.r = r;
		T.g = g;
		T.b = b;
		T.overlay = overlay;
		Return T;
	EndMethod
EndType
